@@ -1,3 +1,5 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 import os
@@ -7,12 +9,13 @@ from calibre import prints
 from calibre.constants import iswindows
 from calibre.customize import Plugin
 
+
 class DevicePlugin(Plugin):
     """
     Defines the interface that should be implemented by backends that
-    communicate with an ebook reader.
+    communicate with an e-book reader.
     """
-    type = _('Device Interface')
+    type = _('Device interface')
 
     #: Ordered list of supported formats
     FORMATS     = ["lrf", "rtf", "pdf", "txt"]
@@ -38,11 +41,6 @@ class DevicePlugin(Plugin):
 
     #: Height for thumbnails on the device
     THUMBNAIL_HEIGHT = 68
-    #: Width for thumbnails on the device. Setting this will force thumbnails
-    #: to this size, not preserving aspect ratio. If it is not set, then
-    #: the aspect ratio will be preserved and the thumbnail will be no higher
-    #: than THUMBNAIL_HEIGHT
-    # THUMBNAIL_WIDTH = 68
 
     #: Compression quality for thumbnails. Set this closer to 100 to have better
     #: quality thumbnails with fewer compression artifacts. Of course, the
@@ -97,7 +95,7 @@ class DevicePlugin(Plugin):
     #: managing a blacklist of devices, a list of ejected devices and so forth.
     #: calibre will periodically call the detect_managed_devices() method and
     #: if it returns a detected device, calibre will call open(). open() will
-    #: be called every time a device is returned even is previous calls to open()
+    #: be called every time a device is returned even if previous calls to open()
     #: failed, therefore the driver must maintain its own blacklist of failed
     #: devices. Similarly, when ejecting, calibre will call eject() and then
     #: assuming the next call to detect_managed_devices() returns None, it will
@@ -234,7 +232,7 @@ class DevicePlugin(Plugin):
         is capable of handling it. If it is not it should return False. This method
         is only called after the vendor, product ids and the bcd have matched, so
         it can do some relatively time intensive checks. The default implementation
-        returns True. This method is called only on windows. See also
+        returns True. This method is called only on Windows. See also
         :meth:`can_handle`.
 
         Note that for devices based on USBMS this method by default delegates
@@ -255,6 +253,7 @@ class DevicePlugin(Plugin):
         '''
 
         return True
+    can_handle.is_base_class_implementation = True
 
     def open(self, connected_device, library_uuid):
         '''
@@ -375,10 +374,10 @@ class DevicePlugin(Plugin):
 
     def books(self, oncard=None, end_session=True):
         """
-        Return a list of ebooks on the device.
+        Return a list of e-books on the device.
 
-        :param oncard:  If 'carda' or 'cardb' return a list of ebooks on the
-                        specific storage card, otherwise return list of ebooks
+        :param oncard:  If 'carda' or 'cardb' return a list of e-books on the
+                        specific storage card, otherwise return list of e-books
                         in main memory of device. If a card is specified and no
                         books are on the card return empty list.
 
@@ -426,7 +425,7 @@ class DevicePlugin(Plugin):
                           :meth`books(oncard='cardb')`).
 
         '''
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def delete_books(self, paths, end_session=True):
         '''
@@ -538,7 +537,7 @@ class DevicePlugin(Plugin):
 
     def startup(self):
         '''
-        Called when calibre is is starting the device. Do any initialization
+        Called when calibre is starting the device. Do any initialization
         required. Note that multiple instances of the class can be instantiated,
         and thus __init__ can be called multiple times, but only one instance
         will have this method called. This method is called on the device
@@ -700,7 +699,7 @@ class DevicePlugin(Plugin):
         name is ignored in cases where the device uses a template to generate
         the file name, which most do. The second value in the returned tuple
         indicated whether the format is future-dated. Return True if it is,
-        otherwise return False. Calibre will display a dialog to the user
+        otherwise return False. calibre will display a dialog to the user
         listing all future dated books.
 
         Extremely important: this method is called on the GUI thread. It must
@@ -712,6 +711,7 @@ class DevicePlugin(Plugin):
         '''
         return (None, (None, False))
 
+
 class BookList(list):
     '''
     A list of books. Each Book object must have the fields
@@ -766,6 +766,7 @@ class BookList(list):
         '''
         raise NotImplementedError()
 
+
 class CurrentlyConnectedDevice(object):
 
     def __init__(self):
@@ -775,6 +776,7 @@ class CurrentlyConnectedDevice(object):
     def device(self):
         return self._device
 
+
 # A device driver can check if a device is currently connected to calibre using
 # the following code::
 #   from calibre.device.interface import currently_connected_device
