@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 __license__ = 'GPL 3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
@@ -16,13 +17,14 @@ class PDBOutput(OutputFormatPlugin):
     name = 'PDB Output'
     author = 'John Schember'
     file_type = 'pdb'
+    commit_name = 'pdb_output'
+    ui_data = {'formats': tuple(ALL_FORMAT_WRITERS)}
 
-    options = set([
+    options = {
         OptionRecommendation(name='format', recommended_value='doc',
             level=OptionRecommendation.LOW,
             short_switch='f', choices=list(ALL_FORMAT_WRITERS),
-            help=(_('Format to use inside the pdb container. Choices are:')+
-            ' %s' % sorted(ALL_FORMAT_WRITERS))),
+            help=(_('Format to use inside the pdb container. Choices are:') + ' %s' % sorted(ALL_FORMAT_WRITERS))),
         OptionRecommendation(name='pdb_output_encoding', recommended_value='cp1252',
             level=OptionRecommendation.LOW,
             help=_('Specify the character encoding of the output document. '
@@ -31,15 +33,15 @@ class PDBOutput(OutputFormatPlugin):
         OptionRecommendation(name='inline_toc',
             recommended_value=False, level=OptionRecommendation.LOW,
             help=_('Add Table of Contents to beginning of the book.')),
-    ])
+    }
 
     def convert(self, oeb_book, output_path, input_plugin, opts, log):
         close = False
         if not hasattr(output_path, 'write'):
             close = True
-            if not os.path.exists(os.path.dirname(output_path)) and os.path.dirname(output_path) != '':
+            if not os.path.exists(os.path.dirname(output_path)) and os.path.dirname(output_path):
                 os.makedirs(os.path.dirname(output_path))
-            out_stream = open(output_path, 'wb')
+            out_stream = lopen(output_path, 'wb')
         else:
             out_stream = output_path
 
@@ -60,4 +62,3 @@ class PDBOutput(OutputFormatPlugin):
 
         if close:
             out_stream.close()
-
