@@ -1,12 +1,12 @@
 #!/usr/bin/env python2
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+from __future__ import absolute_import, division, print_function, unicode_literals
 
 import re, sys
 from collections import defaultdict
 
-from lxml.etree import tostring
+from polyglot.builtins import reraise, unicode_type
+
 from lxml.html import (fragment_fromstring, document_fromstring,
         tostring as htostring)
 
@@ -15,7 +15,7 @@ from calibre.ebooks.readability.cleaners import html_cleaner, clean_attributes
 
 
 def tounicode(tree_or_node, **kwargs):
-    kwargs['encoding'] = unicode
+    kwargs['encoding'] = unicode_type
     return htostring(tree_or_node, **kwargs)
 
 
@@ -62,7 +62,7 @@ def to_int(x):
 
 
 def clean(text):
-    text = re.sub('\s*\n\s*', '\n', text)
+    text = re.sub('\\s*\n\\s*', '\n', text)
     text = re.sub('[ \t]{2,}', ' ', text)
     return text.strip()
 
@@ -154,9 +154,9 @@ class Document:
                     continue  # try again
                 else:
                     return cleaned_article
-        except StandardError, e:
+        except Exception as e:
             self.log.exception('error getting summary: ')
-            raise Unparseable(str(e)), None, sys.exc_info()[2]
+            reraise(Unparseable, Unparseable(str(e)), sys.exc_info()[2])
 
     def get_article(self, candidates, best_candidate):
         # Now that we have the top candidate, look through its siblings for content that might also be related.
@@ -183,7 +183,7 @@ class Document:
 
                 if node_length > 80 and link_density < 0.25:
                     append = True
-                elif node_length < 80 and link_density == 0 and re.search('\.( |$)', node_content):
+                elif node_length < 80 and link_density == 0 and re.search(r'\.( |$)', node_content):
                     append = True
 
             if append:
@@ -313,7 +313,7 @@ class Document:
     def transform_misused_divs_into_paragraphs(self):
         for elem in self.tags(self.html, 'div'):
             # transform <div>s that do not contain other block elements into <p>s
-            if not REGEXES['divToPElementsRe'].search(unicode(''.join(map(tostring, list(elem))))):
+            if not REGEXES['divToPElementsRe'].search(unicode_type(''.join(map(tounicode, list(elem))))):
                 # self.debug("Altering %s to p" % (describe(elem)))
                 elem.tag = "p"
                 # print "Fixed element "+describe(elem)
@@ -445,7 +445,7 @@ class Document:
                         # self.debug(sib.text_content())
                         sib_content_length = text_length(sib)
                         if sib_content_length:
-                            i =+ 1
+                            i += 1
                             siblings.append(sib_content_length)
                             if i == x:
                                 break
@@ -502,10 +502,11 @@ def main():
     enc = sys.__stdout__.encoding or 'utf-8'
     if options.verbose:
         default_log.filter_level = default_log.DEBUG
-    print (Document(raw, default_log,
+    print(Document(raw, default_log,
             debug=options.verbose,
             keep_elements=options.keep_elements).summary().encode(enc,
                 'replace'))
 
+
 if __name__ == '__main__':
     main()
